#include "mbed.h"
#include "w6v6a.h"

extern DigitalOut motor_sw1;
extern DigitalOut motor_dir1;
extern DigitalOut motor_sw2;
extern DigitalOut motor_dir2;
extern DigitalOut power_sw;

extern PwmOut motor_pwm1;
extern PwmOut motor_pwm2;
extern PwmOut servo_pwm1;

extern AnalogIn psd_F;
extern AnalogIn Battery;
extern AnalogIn psd_L;

extern DigitalIn wheel_R;
extern DigitalIn wheel_L;

extern Timer timer;

extern Serial pc;
extern Serial h8;
extern DigitalOut myled;

unsigned int read_ii(Serial *port) {
    unsigned int    data;
    unsigned int    data_L, data_H;

    while (!port->readable());
    data_H = port->getc();

    while (!port->readable());
    data_L = port->getc();

    data = data_H * 256 + data_L;
    return(data);
}

void print_ii(Serial *port, unsigned int data) {
    unsigned int    ii;
    unsigned char    data_L, data_H;

    if (data > 255) {
        ii = data / 256;
        data_H = (unsigned char)ii;
    } else {
        ii = 0;
        data_H = 0;
    }

    ii = data - ii * 256;
    data_L = (unsigned char)ii;

    while (!port->writeable());
    port->putc(data_H);

    while (!port->writeable());
    port->putc(data_L);

    return;
}

void sio_comm1(unsigned char command, struct SIO *sio) {
    unsigned char   i, work_c, command2;
    unsigned int    ii, servo_data[9];
    float           pwm_L, pwm_R;

    //timer range testing(how long the timer can hold the time passed.)
    if (command == 't') {
        printf("timer_f = %8.2f\r\n", timer.read());
        printf("\r\n");
    }

    //Show the present wheel encorder couter value.
    if (command == 'v') {
        printf("W_R = %5d\r\n", sio->count_R);
        printf("W_L = %5d\r\n", sio->count_L);
        printf("\r\n");
    }

    //Show the present speed.
    if (command == 's') {
        printf("Speed_R = %5.2f\r\n", sio->speed_R);
        printf("Speed_L = %5.2f\r\n", sio->speed_L);
        printf("\r\n");
    }

    //Reset the wheel encorder couter value.
    if (command == 'r') {
        sio->count_R = 0;
        sio->count_L = 0;
    }

    //get angle data for the neck joints.
    //The newtral value is 3000(1.5m sec puls width).
    if (command == 'A') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        ii = read_ii(&pc);
        servo_pwm1 = sio->servo_newtral + ((float)ii - 2170.0) / 2170.0 * 0.5;
    }

    //get angle data for the arms joints.
    if (command == 'B') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //arm servo
        for (i = 0; i < 9; i++) {
            servo_data[i] = read_ii(&pc);
        }

        //transmit command action to h8
        while (!h8.writeable());
        h8.putc(command);
        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        //arm servo
        for (i = 0; i < 8; i++) {
            print_ii(&h8, servo_data[i]);
        }
        //neck servo
        servo_pwm1 = sio->servo_newtral + ((float)servo_data[8] - 2170.0) / 2170.0 * 0.05;
    }

    //Transmit 'K' command to No.2 sub CPU.
    //RC servo on/off command
    if (command == 'K') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //get second command from PC
        while (!pc.readable());
        command2 = pc.getc();

        //transmit "K" command action to h8
        //write K command
        while (!h8.writeable());
        h8.putc(command);

        //get ack
        while (!h8.readable());
        work_c = h8.getc();

        if (work_c == 'K') {
            //write servo switch value
            while (!h8.writeable());
            h8.putc(command2);

            //neck servo pwm control
            if ((command2 & 1) == 1) {
                servo_pwm1 = sio->servo_newtral;
            } else {
                servo_pwm1 = 0.0;
            }
        }
    }


    //Battery voltage report
    if (command == 'M') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        while (!pc.writeable());
        sio->Battery = Battery * 3.3 * 3.0;
        pc.printf("%5.1f\r\n",sio->Battery);
    }

    //Stop moving with space or enter key or '0'command
    if (command == 32 || command == 13 || command == '0') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        motor_sw1 = 0;
        motor_sw2 = 0;

        sio->count_R = 0;
        sio->count_L = 0;
        
        sio->running_mode = 0;
    }

    //move command
    if (command == '3') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        //wheel direction
        while (!pc.readable());
        work_c = pc.getc();
        if (work_c == 'F') {
            sio->rot_dir_L = 0;
        } else {
            sio->rot_dir_L = 1;
        }
        work_c = pc.getc();
        if (work_c == 'F') {
            sio->rot_dir_R = 0;
        } else {
            sio->rot_dir_R = 1;
        }

        //speed
        work_c = pc.getc();
        sio->speed_target_L = (float)work_c / 10.0;
        work_c = pc.getc();
        sio->speed_target_R = (float)work_c / 10.0;
        
        sio->speed_target = (sio->speed_target_L + sio->speed_target_R) / 2.0;

        //distance
        sio->count_Lt = read_ii(&pc);
        sio->count_Rt = read_ii(&pc);

        //****************************************
        //running counter reset
        sio->count_L = 0;
        sio->count_R = 0;

        //start wheel motors
        motor_pwm1 = 0.50; // right side motor
        motor_dir1 = sio->rot_dir_R;
        if (sio->count_Rt != 0) motor_sw1 = 1;

        motor_pwm2 = 0.50; // left side motor
        motor_dir2 = sio->rot_dir_L;
        if (sio->count_Lt != 0) motor_sw2 = 1;

        //limitted distance mode
        sio->running_mode = 1;
        timer.reset();
        timer.start();
    }

    //report running status
    if (command == '6') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        while (!pc.writeable());
        pc.printf("%d\r\n",sio->running_mode);
        pc.printf("%d\r\n",sio->count_R);
        pc.printf("%d\r\n",sio->count_L);
        pc.printf("%3.1f\r\n",sio->speed_R);
        pc.printf("%3.1f\r\n",sio->speed_L);
        pc.printf("%4.2f\r\n",motor_pwm1.read());
        pc.printf("%4.2f\r\n",motor_pwm2.read());
    }

    //report battery voltage
    if (command == '7') {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        while (!pc.writeable());
        sio->psd_volt = psd_F * 3.3;
        pc.printf("%5.1f\r\n",sio->psd_volt);
    }

    //functional evaluation
    if (command == '9') {
        ii = read_ii(&pc);
        printf("read_ii = %d\r\n", ii);
    }

    //**********************************************
    //Remote Controle commands with cursor keys.
    if (command == 27) {
        //Send back ack
        while (!pc.writeable());
        pc.putc(command);

        while (!pc.readable());
        command = pc.getc();// 91---fixed charactor
        while (!pc.readable());
        command = pc.getc();//sub command

        pwm_R = motor_pwm1;
        pwm_L = motor_pwm2;
        
        //forward
        if (command == 65) {
            if(motor_sw1 == 0 && motor_sw2 == 0){
                motor_dir1 = 0;
                motor_dir2 = 0;
                motor_sw1 = 1;
                motor_sw2 = 1;
                pwm_R = 0.6;
                pwm_L = 0.6;
            }else{
                //pwm_R = (pwm_R + pwm_L) / 2.0;
                //pwm_L = pwm_R;
                pwm_R = pwm_R - 0.15;
                pwm_L = pwm_L - 0.15;
                if(pwm_R < 0.0) pwm_R = 0.0;
                if(pwm_L < 0.0) pwm_L = 0.0;
            }
        }

        //right
        if (command == 67) {
            if(motor_sw1 == 0 && motor_sw2 == 0){
                motor_dir1 = 1;
                motor_dir2 = 0;
                motor_sw1 = 0;
                motor_sw2 = 1;
                pwm_R = 0.6;
                pwm_L = 0.6;
            }else{
                pwm_R = pwm_R + 0.15;
                pwm_L = pwm_L - 0.15;
                if(pwm_R > 0.75) pwm_R = 0.75;
                if(pwm_L < 0.0) pwm_L = 0.0;
            }
        }

        //left
        if (command == 68) {
            if(motor_sw1 == 0 && motor_sw2 == 0){
                motor_dir1 = 0;
                motor_dir2 = 1;
                motor_sw1 = 1;
                motor_sw2 = 0;
                pwm_R = 0.6;
                pwm_L = 0.6;
            }else{
                pwm_R = pwm_R - 0.15;
                pwm_L = pwm_L + 0.15;
                if(pwm_R < 0.0) pwm_R = 0.0;
                if(pwm_L > 0.75) pwm_L = 0.75;
            }
        }

        //back
        if (command == 66) {
            //go backward in case stopping
            if(motor_sw1 == 0 && motor_sw2 == 0){
                motor_dir1 = 1;
                motor_dir2 = 1;
                motor_sw1 = 1;
                motor_sw2 = 1;
                pwm_R = 0.3;
                pwm_L = 0.3;
            }else{
                //slow down
                pwm_R = pwm_R + 0.15;
                pwm_L = pwm_L + 0.15;
                if(pwm_R > 0.75) pwm_R = 0.75;
                if(pwm_L > 0.75) pwm_L = 0.75;
            }
        }

        motor_pwm1 = pwm_R;
        motor_pwm2 = pwm_L;
        sio->running_mode = 0;
    }
}
