#include "mbed.h"
#include "w6v6a.h"

DigitalOut motor_sw1(p5);
DigitalOut motor_dir1(p6);
DigitalOut motor_sw2(p7);
DigitalOut motor_dir2(p8);
DigitalOut power_sw(p30);

PwmOut motor_pwm1(p21);//right motor
PwmOut motor_pwm2(p22);//left motor
PwmOut servo_pwm1(p23);//neck servo

AnalogIn psd_F(p15);
AnalogIn Battery(p16);
AnalogIn psd_L(p17);

DigitalIn wheel_R(p11);
DigitalIn wheel_L(p12);

Timer timer;

Serial pc(USBTX, USBRX); // tx, rx 9600bps default, port connected to PC
Serial h8(p13, p14); // tx, rx 9600bps default, Port connected to H8 micro controller
DigitalOut myled(LED1);

int main() {
    unsigned char command, W_R, W_L;
    unsigned int count_R1, count_L1;
    float a, b, c, count_diff;
    float pwm_min, pwm_max, pwm_delta, pwm_gain, time_valf;
    float speed_feedback_interval;

    struct SIO sio;

    pc.baud(38400);
    //pc.printf("Control program for w6v4a robot mobile unit.\r\n");

    h8.baud(38400);

    sio.speed_target_R = 0.0;
    sio.speed_target_L = 0.0;
    //Right motor setting
    motor_pwm1.period_ms(10);
    motor_pwm1.write(0.4);//smaller number make the robot faster
    motor_dir1 = 0;
    motor_sw1 = 0;
    //Left motor setting
    motor_pwm2.period_ms(10);
    motor_pwm2.write(0.4);
    motor_dir2 = 0;
    motor_sw2 = 0;

    servo_pwm1.period_ms(20);
    sio.servo_newtral = 1.5 / 20.0;

    //Pwm pulse for the neck servo for initial status.
    servo_pwm1 = 0.0;

    //Switch the DIP relay for PSD & rotaly encoder.
    power_sw = 1;

    W_R = 0;
    W_L = 0;
    sio.count_R = 0;//latest
    sio.count_L = 0;
    sio.count_Rt = 0;//target counter value
    sio.count_Lt = 0;
    count_R1 = 0;//last
    count_L1 = 0;

    pwm_min = 0.0;
    pwm_max = 0.85;
    pwm_delta = (pwm_max - pwm_min) / 128.0;
    //pwm_gain = 5.0;
    pwm_gain = 8.0;

    sio.speed_target_R = 1.20;// km/h
    sio.speed_target_L = 1.20;// km/h
    
    speed_feedback_interval = 0.3;//second

    sio.running_mode = 0; // 0: free running 1:distance limitted

    while (1) {
        wait_ms(5);//wait to prevent chatta signal
        command = 0;
        if (pc.readable()) {
            command = pc.getc();
            sio_comm1(command, &sio);
            //pc.printf("command = %d\r\n", command);
            //pc.putc(command);
            //pc.printf("\r\n");
        }

        //***********************************************************
        //Wheel encorder status watching.
        if (W_R !=  wheel_R) {
            W_R = wheel_R;
            sio.count_R++;
            if (sio.count_R >= sio.count_Rt && sio.running_mode == 1) {
                motor_sw1 = 0;
                count_R1 = 0;//last

                //Case in large radius turn
                if (sio.count_Rt != sio.count_Lt) {
                    motor_sw2 = 0;
                    count_L1 = 0;
                }
                if (motor_sw2.read() == 0){
                     sio.running_mode = 0;
                     timer.stop();
                }
            }
        }
        if (W_L !=  wheel_L) {
            W_L = wheel_L;
            sio.count_L++;
            if (sio.count_L >= sio.count_Lt && sio.running_mode == 1) {
                motor_sw2 = 0;
                count_L1 = 0;
                
                //Case in large radius turn
                if (sio.count_Rt != sio.count_Lt) {
                    motor_sw1 = 0;
                    count_R1 = 0;//last
                }
                if (motor_sw1.read() == 0){
                     sio.running_mode = 0;
                     timer.stop();
                }
            }
        }
        //***********************************************************
        //Speed calculation
        time_valf = timer.read();
        if (time_valf > speed_feedback_interval && sio.running_mode == 1) {
            if (motor_sw1.read() == 1) {
                sio.speed_R = (float)(sio.count_R - count_R1) * 8.11577 / time_valf * 0.0036;// km/h
                count_R1 = sio.count_R;
            }
            if (motor_sw2.read() == 1) {
                sio.speed_L = (float)(sio.count_L - count_L1) * 8.11577 / time_valf * 0.0036;
                count_L1 = sio.count_L;
            }

            timer.reset();
            timer.start();
            
            //syncronze when going strait
            if(sio.count_Lt == sio.count_Rt && sio.running_mode == 1){
                b = (float)sio.count_L;
                c = (float)sio.count_R;
                count_diff = b - c;
                a = fabs(count_diff * 0.02);
                if (a >  1.0) a =  1.0;
                
                if(sio.speed_target > 1.2){
                    if(sio.count_L > sio.count_R){
                        sio.speed_target_L = sio.speed_target - a;
                    }else{
                        sio.speed_target_R = sio.speed_target - a;
                    }
                }else{
                    if(sio.count_L > sio.count_R){
                        sio.speed_target_R = sio.speed_target + a;
                    }else{
                        sio.speed_target_L = sio.speed_target + a;
                    }
                }
            }

            //Speed feedback
            if (motor_sw1.read() == 1) {
                a = motor_pwm1.read() - (sio.speed_target_R - sio.speed_R) * pwm_gain * pwm_delta;
                if (a < pwm_min) a = pwm_min;
                if (a > pwm_max) a = pwm_max;
                motor_pwm1.write(a);
            }


            if (motor_sw2.read() == 1) {
                a = motor_pwm2.read() - (sio.speed_target_L - sio.speed_L) * pwm_gain * pwm_delta;
                if (a < pwm_min) a = pwm_min;
                if (a > pwm_max) a = pwm_max;
                motor_pwm2.write(a);
            }
        }

    }
}
